#ifndef __NQS_GENERIC_LOCK_HEADER
#define __NQS_GENERIC_LOCK_HEADER

/*
 * nqs_generic/all-systems/lock.h
 * Pick up header file for plock() support
 */

#include <nqs_generic/license.h>
  /* license information */
#include <sys/lock.h>

#endif /* __NQS_GENERIC_LOCK_HEADER */
