#ifndef __NQS_DB_PROTO_HEADER
#define __NQS_DB_PROTO_HEADER

/*
 * nqs_db/all-systems/proto.h
 * Function prototypes.
 */

#include <nqs_generic/license.h>
  /* NQS license information */
#include <grp.h>
#include <sys/types.h>
  /* for struct group type definition, and gid_t */
#include <pwd.h>
  /* for struct passwd */

/*
 * function prototypes
 */

extern struct group *	fetchgrgid	(gid_t);
extern struct passwd *	fetchpwuid	(uid_t);

#endif /* __NQS_DB_PROTO_HEADER */
