/*
 * nqs_generic/all-systems/mergertcm.c
 * Merge a TCM code with an RCM code ...
 */

#include <nqs_generic/license.h>
  /* NQS license information */
#include <nqs_generic/debug.h>
  /* debugging support */
#include <nqs_generic/XCI.h>
  /* Completion code information bits */
#include <nqs_generic/proto.h>
  /* ANSI C prototypes */

/*** mergertcm
 *
 *	long mergertcm():
 *	Merge TCMx_ code with RCM_code.
 */
long mergertcm (long rcm_original, long tcm_original)
{
  ENTER_FUNCTION("nqs_generic/all-systems/mergertcm");
    /* debugging info - probably overkill here ... */
  EXIT_FUNCTION;
  return (rcm_original | ((tcm_original & XCI_FULREA_MASK) << 10));
}
