/*
 * nqs_generic/all-systems/asciierrno.c
 * Format present errno value in ASCII diagnostic string.
 */

#include <nqs_generic/license.h>
  /* NQS license information */
#include <nqs_generic/debug.h>
  /* debugging information */
#include <nqs_generic/proto.h>
  /* ANSI C prototypes */

#include <stdio.h>
#include <errno.h>

/*** asciierrno
 *
 *
 *	char *asciierrno():
 *	Return formatted ASCII value of present errno.
 */
char *asciierrno(void)
{
  static char szFormatBuffer [20];
  ENTER_FUNCTION("nqs_generic/all-systems/asciierrno");

  sprintf (szFormatBuffer, "Errno value: %1d", errno);
  
  EXIT_FUNCTION;
  
  return (szFormatBuffer);
}
