#ifndef __NQS_REQ_HEADER
#define __NQS_REQ_HEADER

/*
 * nqs_req/all-systems/nqs_req.h
 * Header file for NQS request library
 * 
 * Source code outside libnqs_req wishing to use this library should
 * include this header file.
 * 
 * Source code inside libnqs_req wishing to use this library should include
 * individual header files as required.
 */

#include <nqs_generic/license.h>
  /* NQS license information */

#include <nqs_req/proto.h>
  /* function prototypes */

#endif /* __NQS_REQ_HEADER */
