#ifndef __NQS_QUEUE_PRM_HEADER
#define __NQS_QUEUE_PRM_HEADER

/*
 * nqs_queue/all-systems/PRM.h
 * Parameter record types for Monsanto-NQS
 */

#include <nqs_generic/license.h>

/*
 *	NQS general parameter record TYPES (NOT values).
 */
#define	PRM_GENPARAMS	0		/* General NQS operating parameters */
#define	PRM_LOGFILE	1		/* Logfile parameter */
#define	PRM_NETPROCS	2		/* Network client, Network server */
#define	PRM_NETPEER	3		/* Network peer configuration */

#endif /* __NQS_QUEUE_PRM_HEADER */
