#ifndef __NQS_QUEUE_RTYPE_HEADER
#define __NQS_QUEUE_RTYPE_HEADER

/*
 * nqs_queue/all-systems/RTYPE.h
 * Request type specifiers for Monsanto-NQS
 */

#include <nqs_generic/license.h>

/*
 *	NQS request type specifiers.
 */
#define	RTYPE_BATCH	1		/* Batch request */
#define	RTYPE_DEVICE	2		/* Device oriented request */

#endif /* __NQS_QUEUE_RTYPE_HEADER */
