/*
 * nqs_generic/all-systems/bufstderr.c
 * Set the buffer size for the stderr stream.
 */

#include <nqs_generic/license.h>
  /* NQS license information */
#include <nqs_generic/debug.h>
  /* debugging support */
#include <nqs_generic/proto.h>
  /* ANSI C prototypes */

#include <stdio.h>

/*** bufstderr
 *
 *	void bufstderr():
 *	Block buffer the output to the stderr file stream.
 */
void bufstderr(void)
{
  static char stderr_buffer [BUFSIZ];	/* Stderr output buffer */

  ENTER_FUNCTION("nqs_generic/all-systems/bufstderr");
  setvbuf (stderr, stderr_buffer, _IOFBF, BUFSIZ);
  EXIT_FUNCTION;
}
