#ifndef __NQS_REQ_PROTO_HEADER
#define __NQS_REQ_PROTO_HEADER

/*
 * nqs_req/all-systems/proto.h
 * Function prototypes
 */

#include <nqs_generic/license.h>
  /* NQS license information */
#include <nmap/nmap.h>
  /* for Mid_t type */

/*
 * function prototypes
 */

extern int	opendata	(long, Mid_t, int);

#endif /* __NQS_REQ_PROTO_HEADER */
