#ifndef __NQS_QUEUE_TRA_HEADER
#define __NQS_QUEUE_TRA_HEADER

/*
 *	Transaction types.
 *	WARNING:  These values must be in [0..3].
 */
#define	TRA_UNALLOCATED	0		/* Transaction descriptor is */
					/* unallocated */
#define	TRA_REQSTATE	1		/* Transaction descriptor is */
					/* allocated, and describes the */
					/* state of a request */

#endif /* __NQS_QUEUE_TRA_HEADER */
