#ifndef __NQS_COMMS_NQS_FIFO_X_HEADER
#define __NQS_COMMS_NQS_FIFO_X_HEADER

/*
 * nqs_comms/BSD/nqs_fifo_x.h
 * Directories for FIFO.
 */

#include <nqs_generic/license.h>
  /* NQS license information */

/*
 * No directories are defined, because BSD UNIX does not support named pipes
 */

#endif /* __NQS_COMMS_NQS_FIFO_X_HEADER */
