#ifndef __NQS_COMMS_NQS_FIFO_X_HEADER
#define __NQS_COMMS_NQS_FIFO_X_HEADER

/*
 * nqs_comms/all-systems/nqs_fifo_x.h
 * Directory definitions for the FIFO used for local communication
 */

#include <nqs_generic/license.h>
  /* NQS license information */

extern char *Nqs_fifo;
  /* NQS interprocess FIFO pipe relative pathname */
extern char *Nqs_ffifo;
  /* NQS interprocess FIFO pipe full pathname */

#endif /* __NQS_COMMS_NQS_FIFO_X_HEADER */
