#ifndef __NQS_DB_HEADER
#define __NQS_DB_HEADER

/*
 * nqs_db/all-systems/nqs_db.h
 * Header file for the nqs_db library.
 * 
 * All source code outside this library wishing to use this library should
 * include this header file ALONE.
 * 
 * All source code within this library wishing to use this library should
 * include any individual header files required.
 */

#include <nqs_generic/license.h>
  /* NQS license information */

#include <nqs_db/proto.h>
  /* ANSI C prototypes for the library */

#endif /* __NQS_DB_HEADER */
