#ifndef __NQS_COMMS_A6BTOUL_HEADER
#define __NQS_COMMS_A6BTOUL_HEADER

/*
 * nqs_comms/all-systems/a6btoul.h
 */

#include <nqs_generic/license.h>

/*
 * function prototypes
 */

unsigned long a6btoul (char *, int);
  /* convert a six-bit string into an unsigned long */
int is6bitstr (char *, int);
  /* check whether a string is 6-bit or not */
#endif /* __NQS_COMMS_A6BTOUL_HEADER */
