/*
 * nqs_generic/BSD/bufstdout.c
 * Set the buffer size on the stdout stream.
 */

#include <nqs_generic/license.h>
  /* NQS license information */
#include <nqs_generic/debug.h>
  /* debugging support */

#include <stdio.h>

/*** bufstdout
 *
 *	void bufstdout():
 *	Block buffer the output to the stdout file stream.
 */
void bufstdout(void)
{
  static char szStdout_buffer [BUFSIZ];	/* Stdout output buffer */

  ENTER_FUNCTION("nqs_generic/BSD/bufstdout");
  setbuffer (stdout, szStdout_buffer, BUFSIZ);
  EXIT_FUNCTION;
}
