#ifndef __NQS_QUEUE_RQS_HEADER
#define __NQS_QUEUE_RQS_HEADER

/*
 * nqs_queue/all-systems/RQS.h
 * Request queue states for Monsanto-NQS
 */

#include <nqs_generic/license.h>

/*
 *	NQS request queue states (RQS_).
 */
#define	RQS_DEPARTING	0000001		/* Departing/staging-out state */
#define	RQS_RUNNING	0000002		/* Running state */
#define	RQS_STAGING	0000004		/* Staging-in state */
#define	RQS_QUEUED	0000010		/* Queued state */
#define	RQS_WAITING	0000020		/* Waiting state */
#define	RQS_HOLDING	0000040		/* Holding state */
#define	RQS_ARRIVING	0000100		/* Arriving state */

#endif /* __NQS_QUEUE_RQS_HEADER */
