#ifndef __NQS_QUEUE_MEMDUMP_HEADER
#define __NQS_QUEUE_MEMDUMP_HEADER

/*
 * nqs_queue/all-systems/MEMDUMP.h
 * Memory dump flags for Monsanto-NQS
 */

#include <nqs_generic/license.h>

/*
 * Memdump flags
 */
#define MEMDUMP_NONE      0		/* No dump flags */
#define MEMDUMP_REQ       1		/* Dump the request data structs */
#define MEMDUMP_LOAD      2		/* Dump the load data structs */

#endif /* __NQS_QUEUE_MEMDUMP_HEADER */
